Ext.namespace('dSS');

dSS.ScenesPanel = Ext.extend(Ext.Panel, {
    initComponent: function() {

        var contentPanel = {
            ref: 'contentPanel',
            region: 'center',
            layout: 'card',
            hideBorders: true,
            border: true,
            activeItem: 0,
            items: [
            {
                xtype: 'dssdevicepanel',
                featureConfig: dSS.config.FeatureConfig.SCENE
            }]
        };

        this.treePanel = new Ext.tree.TreePanel({
            useArrows: true,
            autoScroll: true,
            animate: true,
            enableDD: false,
            containerScroll: true,
            rootVisible: false,
            border: false,
            bodyBorder: false,
            ctCls: 'x-panel-no-top-border',
            root: new Ext.tree.AsyncTreeNode(),
            listeners: {
                click: this.handleNodeClick.createDelegate(this)
            }
        });

        this.treePanel.getRootNode().expand();

        this.apartmentTreePanel = new Ext.tree.TreePanel({
            useArrows: true,
            autoScroll: true,
            animate: true,
            enableDD: false,
            containerScroll: true,
            rootVisible: false,
            border: false,
            bodyBorder: false,
            root: new Ext.tree.AsyncTreeNode({
                children: this.genererateApartmentNodes(),
                id: 'root',
                singleClickExpand: true,
                leaf: false
            }),
            listeners: {
                click: this.handleNodeClick.createDelegate(this)
            }
        });

        this.menuPanel = new Ext.Panel({
            title: dSS._("Scene", "Scenes", 2),
            region: 'west',
            layout: 'border',
            split: true,
            width: 225,
            minSize: 175,
            tbar: this.buildTopToolbar(),
            bodyBorder: false,
            items:
            [
                {
                    xtype: 'panel',
                    ref: 'zoneMenuArea',
                    layout: 'fit',
                    region: 'center',
                    split: true,
                    items:
                    [
                        this.treePanel
                    ]
                },
                {
                    xtype: 'panel',
                    ref: 'apartmentMenuArea',
                    layout: 'fit',
                    region: 'south',
                    //height: 80,
                    split: true,
                    items:
                    [
                        this.apartmentTreePanel
                    ]
                }
            ]
        });

        Ext.apply(this, {
            layout: 'border',
            border: false,
            bodyBorder: false,
            items: [this.menuPanel, contentPanel]
        });

        this.dirtyTree = false;
        this.updateTree();


        dSS.data.Loader.getZoneStore().addListener('datachanged',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.data.Loader.getZoneStore().addListener('update',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.data.Loader.getSceneStore().addListener('datachanged',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.data.Loader.getSceneStore().addListener('update',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.ScenesPanel.superclass.initComponent.apply(this, arguments);

        this.on('afterrender', function() {
                if (dSS.config.firstLoad === true) {
                    var fn =
                        dSS.util.maskViewport.createDelegate(this, [ 'mask' ]);
                    fn();
                    dSS.data.Loader.load();
                    dSS.config.firstLoad = false;
                }
            },
            this);
        this.on('activate', function() {
            if (this.activeNode !== undefined) {
                this.activeNode.setCls();
                this.activeNode = undefined;
            }
            this.handleTreeView();

            if (this.menuSizeSet !== true) {
                var total = this.menuPanel.ownerCt.getHeight();
                this.menuPanel.apartmentMenuArea.setHeight(total / 3);
                this.menuPanel.zoneMenuArea.setHeight((total / 3)*2);
                this.menuPanel.syncSize();
                this.menuSizeSet = true;
            }
        },
        this);

        this.on('redrawtree', this.handleTreeView.createDelegate(this), this,
                { buffer: 1000 });

        dSS.MessageBus.on('scenesmarkdirty',
                           this.markDirty.createDelegate(this), this);

        dSS.MessageBus.on('reloadapartment',
            dSS.util.maskViewport.createDelegate(this, [ 'mask' ]),
            this);

        dSS.MessageBus.on('structureloaded',
            dSS.util.maskViewport.createDelegate(this, [ 'unmask' ]),
            this);

        this.treePanel.getSelectionModel().on('beforeselect', function(sel, node, oldnode) {
                if (node.isLeaf() && (node.disabled === true)) {
                    return false;
                }
                this.apartmentTreePanel.getSelectionModel().clearSelections();
            },
            this);

        this.treePanel.on('contextmenu', this.handleContextMenu, this);
        this.apartmentTreePanel.on('contextmenu', this.handleContextMenu, this);

        this.apartmentTreePanel.getSelectionModel().on('beforeselect',
                function(tree, node) {
                    this.treePanel.getSelectionModel().clearSelections();
                },
                this);

    },
    handleContextMenu: function(node, event) {
            event.preventDefault();
            event.stopEvent();

            if (!node.isSelected()) {
                node.select();
                this.handleNodeClick(node);
            }

            if (!node.isLeaf() || (node.disabled === true)) {
                return;
            }

            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var menuItem = new Ext.menu.Item({
                text: dSS._("Call scene"),
                iconCls: 'callSceneIcon',
                handler: this.processScene.createDelegate(this, [node, false])
            });
            this.contextMenu.add(menuItem);

            if (node.attributes.undo === true) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Undo scene"),
                    iconCls: 'undoSceneIcon',
                    handler: this.processScene.createDelegate(this, [node, true])
                });
                this.contextMenu.add(menuItem);
            }

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                text: dSS._("Save scene"),
                iconCls: 'saveScene',
                handler: this.saveScene.createDelegate(this, [node])
            });
            this.contextMenu.add(menuItem);

            var path = node.getPath().split('/').slice(1);
            if (path.length === 4) {
                var ids = this.getZoneSceneGroup(node.id);
                if (ids.groupId !== null) {
                    menuItem = new Ext.menu.Item({
                        text: dSS._("Rename scene"),
                        iconCls: 'editIcon',
                        handler: this.renameSelectedScene.createDelegate(this,
                                                                         [node])
                    });
                    this.contextMenu.add(menuItem);
                }
            }

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
    markDirty: function() {
        this.dirtyTree = true;
        if (this.isVisible() === true) {
            this.fireEvent('redrawtree');
        }
    },

    handleTreeView: function() {
        if (this.dirtyTree === true) {
            this.updateTree();
            return;
        }

        var selModel = this.treePanel.getSelectionModel();
        var node = selModel.getSelectedNode();
        var root = this.treePanel.getRootNode();

        if (node === null) {
            selModel = this.apartmentTreePanel.getSelectionModel();
            node = selModel.getSelectedNode();
        }
        if (node === null) {
            if (root.hasChildNodes()) {
                root.firstChild.select();
                this.handleNodeClick(root.firstChild);
            } else {
              this.updateTree();
            }
        } else {
            this.handleNodeClick(node);
        }
    },

    getSceneName: function(zone, group, scene) {
        var sceneStore = dSS.data.Loader.getSceneStore();
        var rec = sceneStore.getById(zone + '-' + group + '-' + scene);
        if (rec === undefined) {
            return '';
        }

        var name = rec.get('name');
        if ((name === undefined) || (name.length === 0)) {
            return '';
        }

        return ' - ' + name;
    },

    generateSceneNode: function(zone, group, scene, icon) {
        var node = {};
        var name = dSS.config.sceneNames[scene];
        var tip = dSS.config.sceneTips[scene];
        if (group === dSS.config.groups.SHADOW) {
            switch (scene) {
                case dSS.config.scenes.T0_S0:
                    name = "Down";
                    tip = dSS.config.sceneTipsShadowOverrides[scene];
                    break;
                case dSS.config.scenes.T0_S1:
                    name = "Up";
                    tip = dSS.config.sceneTipsShadowOverrides[scene];
                    break;
                case dSS.config.scenes.T1_S1:
                    name = "Area 1 up";
                    break;
                case dSS.config.scenes.T2_S1:
                    name = "Area 2 up";
                    break;
                case dSS.config.scenes.T3_S1:
                    name = "Area 3 up";
                    break;
                case dSS.config.scenes.T4_S1:
                    name = "Area 4 up";
                    break;
                default:
                    break;
            }
        }

        node.text = dSS._(name) +
                    this.getSceneName(zone, group, scene);
        node.id = zone + '-' + group + '-' + scene;
        node.leaf = true;
        node.iconCls = icon;
        node.qtip = dSS._(tip);
        return node;
    },

    // generates scenes 0-4 for a given zone/group
    generate0to4Scenes: function(zone, group, icon) {
        var addScenes =
        [
            dSS.config.scenes.T0_S0, dSS.config.scenes.T0_S1,
            dSS.config.scenes.T0_S2, dSS.config.scenes.T0_S3,
            dSS.config.scenes.T0_S4
        ];
        var scenes = [];

        var i;
        for (i = 0; i < addScenes.length; i++) {
            var nodeIcon = icon;
            // off scenes have special icons
            if (addScenes[i] === dSS.config.scenes.T0_S0) {
                if (icon.substring(icon.length - 2) === 'On') {
                    nodeIcon = icon.substring(0, icon.length - 2) + 'Off';
                } else {
                    nodeIcon = icon + 'Off';
                }
            }
            scenes.push(
                this.generateSceneNode(zone, group, addScenes[i], nodeIcon));
        }
        return scenes;
    },

    generatePushbuttonScenes: function(zone, group, icon, cfg) {
        var scenes = [];
        var sceneNumbers = [];

        if (this.disableSceneFilter === true) {
            cfg =
            [
                dSS.config.deviceParams.PUSHBUTTON_DEVICE,
                dSS.config.deviceParams.PUSHBUTTON_ZONE0,
                dSS.config.deviceParams.PUSHBUTTON_GROUP1,
                dSS.config.deviceParams.PUSHBUTTON_GROUP2,
                dSS.config.deviceParams.PUSHBUTTON_GROUP3,
                dSS.config.deviceParams.PUSHBUTTON_GROUP4,
                dSS.config.deviceParams.PUSHBUTTON_ZONE1,
                dSS.config.deviceParams.PUSHBUTTON_GROUP5,
                dSS.config.deviceParams.PUSHBUTTON_ZONE2,
                dSS.config.deviceParams.PUSHBUTTON_GROUP6,
                dSS.config.deviceParams.PUSHBUTTON_ZONE3,
                dSS.config.deviceParams.PUSHBUTTON_GROUP7,
                dSS.config.deviceParams.PUSHBUTTON_ZONE4,
                dSS.config.deviceParams.PUSHBUTTON_GROUP8
            ];

        } else {
            cfg = dSS.util.UniqueArray(cfg, true);
            if (cfg.length < 1) {
                return scenes;
            }
        }

        var offScenes =
        [
            dSS.config.scenes.EX5, dSS.config.scenes.EX10,
            dSS.config.scenes.EX15, dSS.config.scenes.EX20
        ];

        var lastScenes =
        [
            dSS.config.scenes.T1_S4, dSS.config.scenes.T2_S4,
            dSS.config.scenes.T3_S4, dSS.config.scenes.T4_S1
        ];

        var i;
        for (i = 0; i < cfg.length; i++) {
            switch (cfg[i]) {
                case dSS.config.deviceParams.PUSHBUTTON_DEVICE:
                    // device pushbutton - no extra nodes
                case dSS.config.deviceParams.PUSHBUTTON_ZONE0:
                    // zone pushbutton, defaults scenes 0-4, no extra nodes
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
                    // mood area 1
                    if (this.disableSceneFilter === true) {
                        offScenes.push(dSS.config.scenes.T1_S0);
                        sceneNumbers.push(dSS.config.scenes.T1_S0);
                    }
                    sceneNumbers.push(dSS.config.scenes.T1_S1);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
                    // mood area 2
                    if (this.disableSceneFilter === true) {
                        offScenes.push(dSS.config.scenes.T2_S0);
                        sceneNumbers.push(dSS.config.scenes.T2_S0);
                    }
                    sceneNumbers.push(dSS.config.scenes.T2_S1);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
                    // mood area 3
                    if (this.disableSceneFilter === true) {
                        offScenes.push(dSS.config.scenes.T3_S0);
                        sceneNumbers.push(dSS.config.scenes.T3_S0);
                    }
                    sceneNumbers.push(dSS.config.scenes.T3_S1);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
                    // mood area 4
                    if (this.disableSceneFilter === true) {
                        offScenes.push(dSS.config.scenes.T4_S0);
                        sceneNumbers.push(dSS.config.scenes.T4_S0);
                    }
                    sceneNumbers.push(dSS.config.scenes.T4_S1);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
                    // moods 5-9
                    sceneNumbers.push(dSS.config.scenes.EX5);
                    sceneNumbers.push(dSS.config.scenes.EX6);
                    sceneNumbers.push(dSS.config.scenes.T1_S2);
                    sceneNumbers.push(dSS.config.scenes.T1_S3);
                    sceneNumbers.push(dSS.config.scenes.T1_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
                    // moods 7-9
                    sceneNumbers =
                        [ dSS.config.scenes.T1_S1 ].concat(sceneNumbers);
                    sceneNumbers.push(dSS.config.scenes.T1_S2);
                    sceneNumbers.push(dSS.config.scenes.T1_S3);
                    sceneNumbers.push(dSS.config.scenes.T1_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
                    // moods 10-14
                    sceneNumbers.push(dSS.config.scenes.EX10);
                    sceneNumbers.push(dSS.config.scenes.EX11);
                    sceneNumbers.push(dSS.config.scenes.T2_S2);
                    sceneNumbers.push(dSS.config.scenes.T2_S3);
                    sceneNumbers.push(dSS.config.scenes.T2_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
                    // moods 12-14
                    sceneNumbers =
                        [ dSS.config.scenes.T2_S1 ].concat(sceneNumbers);
                    sceneNumbers.push(dSS.config.scenes.T2_S2);
                    sceneNumbers.push(dSS.config.scenes.T2_S3);
                    sceneNumbers.push(dSS.config.scenes.T2_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
                    // moods 15-19
                    sceneNumbers.push(dSS.config.scenes.EX15);
                    sceneNumbers.push(dSS.config.scenes.EX16);
                    sceneNumbers.push(dSS.config.scenes.T3_S2);
                    sceneNumbers.push(dSS.config.scenes.T3_S3);
                    sceneNumbers.push(dSS.config.scenes.T3_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
                    // moods 17-19
                    sceneNumbers =
                        [ dSS.config.scenes.T3_S1 ].concat(sceneNumbers);
                    sceneNumbers.push(dSS.config.scenes.T3_S2);
                    sceneNumbers.push(dSS.config.scenes.T3_S3);
                    sceneNumbers.push(dSS.config.scenes.T3_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
                    // moods 20-24
                    sceneNumbers.push(dSS.config.scenes.EX20);
                    sceneNumbers.push(dSS.config.scenes.EX21);
                    sceneNumbers.push(dSS.config.scenes.T4_S2);
                    sceneNumbers.push(dSS.config.scenes.T4_S3);
                    sceneNumbers.push(dSS.config.scenes.T4_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
                    // moods 22-24
                    sceneNumbers =
                        [ dSS.config.scenes.T4_S1 ].concat(sceneNumbers);
                    sceneNumbers.push(dSS.config.scenes.T4_S2);
                    sceneNumbers.push(dSS.config.scenes.T4_S3);
                    sceneNumbers.push(dSS.config.scenes.T4_S4);
                    break;
                default:
                    break;
            }
        }

        sceneNumbers = dSS.util.UniqueArray(sceneNumbers, true);
        sceneNumbers.sort(function(a, b) {
            return dSS.config.sceneSortOrder[a] - dSS.config.sceneSortOrder[b];
        });

        var len = sceneNumbers.length;

        for (i = 0; i < sceneNumbers.length; i++) {
            var nodeIcon = icon;
            if (offScenes.indexOf(sceneNumbers[i]) !== -1) {
                if (icon.substring(icon.length - 2) === 'On') {
                    nodeIcon = icon.substring(0, icon.length - 2) + 'Off';
                } else {
                    nodeIcon = icon + 'Off';
                }
            }

            var prependSeparator = false;
            var appendSeparator = false;

            switch (sceneNumbers[i]) {
                case dSS.config.scenes.T1_S2:
                    if (sceneNumbers[i - 1] !== dSS.config.scenes.EX6) {
                        prependSeparator = true;
                    }
                    break;
                case dSS.config.scenes.T2_S2:
                    if (sceneNumbers[i - 1] !== dSS.config.scenes.EX11) {
                        prependSeparator = true;
                    }
                    break;
                case dSS.config.scenes.T3_S2:
                    if (sceneNumbers[i - 1] !== dSS.config.scenes.EX16) {
                        prependSeparator = true;
                    }
                    break;
                case dSS.config.scenes.T4_S2:
                    if (sceneNumbers[i - 1] !== dSS.config.scenes.EX21) {
                        prependSeparator = true;
                    }
                    break;
                case dSS.config.scenes.EX5:
                case dSS.config.scenes.EX10:
                case dSS.config.scenes.EX15:
                case dSS.config.scenes.EX20:
                        prependSeparator = true;
                    break;

                default:
                    break;
            }

            if (prependSeparator) {
                scenes.push(this.generateSeparatorNode());
            }
            scenes.push(
                    this.generateSceneNode(zone, group, sceneNumbers[i],
                    nodeIcon));

        }
        return scenes;
    },

    // for leafs the id is encoded as: zone id - group - scene number
    // when no group is desired, 0 is used
    generateSubNodes: function(zoneId, groups) {
        var retNodes = [];
        var childNodes = [];
        if (groups.LIGHT === true) {
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.LIGHT,
                                                 'lightbulbOn');
            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.LIGHT,
                                           'lightbulbOn',
                                           groups.LIGHTpb));
            retNodes.push(
            {
                text: dSS._("Light scenes"),
                id: zoneId + '-' + dSS.config.groups.LIGHT + '-all',
                leaf: false,
                singleClickExpand: true,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.groups.LIGHT],
                children: childNodes
            });
        }

        if (groups.SHADOW === true) {
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.SHADOW,
                                                 'shadesIcon');

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.SHADOW,
                                           'shadesIcon',
                                           groups.SHADOWpb));

            retNodes.push({
                text: dSS._("Shade scenes"),
                id: zoneId + '-' + dSS.config.groups.SHADOW + '-all',
                leaf: false,
                singleClickExpand: true,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.groups.SHADOW],
                children: childNodes
            });
        }

        if (groups.CLIMATE === true) {
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.CLIMATE,
                                                 'climateIcon');

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.CLIMATE,
                                           'climateIcon',
                                           groups.CLIMATEpb));

            retNodes.push({
                text: dSS._("Climate scenes"),
                id: zoneId + '-' + dSS.config.groups.CLIMATE + '-all',
                leaf: false,
                singleClickExpand: true,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.groups.CLIMATE],
                children: childNodes
            });
        }

        if (groups.AUDIO === true) {
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.AUDIO,
                                                 'audioIcon');

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.AUDIO,
                                           'audioIcon',
                                           groups.AUDIOpb));

            retNodes.push({
                text: dSS._("Audio scenes"),
                id: zoneId + '-' + dSS.config.groups.AUDIO + '-all',
                leaf: false,
                singleClickExpand: true,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.groups.AUDIO],
                children: childNodes
            });
        }

        if (groups.VIDEO === true) {
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.VIDEO,
                                                 'videoIcon');

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.VIDEO,
                                           'videoIcon',
                                           groups.VIDEOpb));

            retNodes.push({
                text: dSS._("Video scenes"),
                id: zoneId + '-' + dSS.config.groups.VIDEO + '-all',
                leaf: false,
                singleClickExpand: true,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.groups.VIDEO],
                children: childNodes
            });
        }

        if (groups.WHITE === true) {
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.WHITE,
                                                 'whiteIcon');

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.WHITE,
                                           'whiteIcon',
                                           groups.WHITEpb));

            retNodes.push({
                text: dSS._("White goods scenes"),
                id: zoneId + '-' + dSS.config.groups.WHITE + '-all',
                leaf: false,
                singleClickExpand: true,
                children: childNodes
            });
        }

        retNodes = retNodes.concat(
            [
                {
                    text: dSS._("Standby"),
                    id: zoneId + '-all-' + dSS.config.scenes.STANDBY,
                    leaf: true,
                    iconCls: 'standByIcon'
                },
                {
                    text: dSS._("Deep off"),
                    id: zoneId + '-all-' + dSS.config.scenes.DEEP_OFF,
                    leaf: true,
                    iconCls: 'deepOffIcon'
                },
                {
                    text: dSS._("Sleeping"),
                    id: zoneId + '-all-' + dSS.config.scenes.SLEEPING,
                    leaf: true,
                    iconCls: 'sleepingIcon'
                },
                {
                    text: dSS._("Wake up"),
                    id: zoneId + '-all-' + dSS.config.scenes.WAKE_UP,
                    leaf: true,
                    iconCls: 'wakeupIcon'
                }
            ]
        );

        return retNodes;
    },

    genererateApartmentNodes: function() {
        return [
            {
                text: dSS._("Leave home"),
                id: '0' + '-all-' + dSS.config.scenes.ABSENT,
                leaf: true,
                iconCls: 'leaveHomeIcon'
            },
            {
                text: dSS._("Coming home"),
                id: '0' + '-all-' + dSS.config.scenes.PRESENT,
                leaf: true,
                iconCls: 'comeHomeIcon'
            },
            {
                text: dSS._("Door bell"),
                id: '0' + '-all-' + dSS.config.scenes.SIG_BELL,
                leaf: true,
                iconCls: 'doorBellIcon'
            },
            this.generateSeparatorNode(),
            {
                text: dSS._("Panic"),
                id: '0' + '-all-' + dSS.config.scenes.PANIC,
                iconCls: 'panicIcon',
                undo: true,
                leaf: true
            }
        ];
    },

    generateSeparatorNode: function() {
        return {
            text: '&mdash;&mdash;&mdash;&mdash;&mdash;&mdash;' +
                  '&mdash;&mdash;&mdash;&mdash;&mdash;',
            disabled: true,
            cls: 'x-tree-noicon x-item-disabled mousedefault',
            isTarget: false,
            allowDrag: false,
            allowDrop: false,
            leaf: true,
            listeners: {
                contextmenu: function() { return false; }
            }
        };
    },

    updateTree: function() {
        var devStore = dSS.data.Loader.getDeviceStore();
        devStore.clearFilter();

        var panel = this.apartmentTreePanel;
        var selModel = panel.getSelectionModel();
        var selectedNode = selModel.getSelectedNode();
        var selectedPath = ''

        if (selectedNode === null) {
            panel = this.treePanel;
            selModel = panel.getSelectionModel();
            selectedNode = selModel.getSelectedNode();
        }

        if (selectedNode !== null) {
            selectedPath = selectedNode.getPath();
        }

        var zStore = dSS.data.Loader.getZoneStore();
        var nodes = [];
        zStore.each(
        function(record) {
            if ((record.get('id') !== 0) && (devStore.find('zoneId', record.get('id')) >= 0)) {
                var deviceTypesInZone = {
                    LIGHT:      false,
                    LIGHTpb:    [],
                    SHADOW:     false,
                    SHADOWpb:   [],
                    CLIMATE:    false,
                    CLIMATEpb:  [],
                    AUDIO:      false,
                    AUDIOpb:    [],
                    VIDEO:      false,
                    VIDEOpb:    [],
                    WHITE:      false,
                    WHITEpb:    []
                };
                var addZone = false;
                var pushbuttonConfigs = [];
                devStore.each(function(device) {
                    var pbConfig;
                    if (device.get('zoneId') === record.get('id')) {
                        if (device.get('isPresent') === false) {
                            return;
                        }

                        var buttonID = device.get('buttonID');
                        if ((buttonID !== 0xff) &&
                            (device.get('productRevision') >=
                                                dSS.config.PushbuttonMinFW)) {
                            pbConfig = (buttonID & 0x0000000f);
                        }

                        var fid = dSS.util.parseFunctionID(device.get('functionID'));
                        var joker;

                        switch (fid[0]) {
                            case dSS.config.groups.LIGHT:
                                deviceTypesInZone.LIGHT = true;
                                if (pbConfig !== undefined) {
                                    deviceTypesInZone.LIGHTpb.push(pbConfig);
                                }
                                addZone = true;
                                break;
                            case dSS.config.groups.SHADOW:
                                deviceTypesInZone.SHADOW = true;
                                if (pbConfig !== undefined) {
                                    deviceTypesInZone.SHADOWpb.push(pbConfig);
                                }
                                addZone = true;
                                break;
                            case dSS.config.groups.CLIMATE:
                                deviceTypesInZone.CLIMATE = true;
                                if (pbConfig !== undefined) {
                                    deviceTypesInZone.CLIMATEpb.push(pbConfig);
                                }
                                addZone = true;
                                break;
                            case dSS.config.groups.AUDIO:
                                deviceTypesInZone.AUDIO = true;
                                if (pbConfig !== undefined) {
                                    deviceTypesInZone.AUDIOpb.push(pbConfig);
                                }
                                addZone = true;
                                break;
                            case dSS.config.groups.VIDEO:
                                deviceTypesInZone.VIDEO = true;
                                if (pbConfig !== undefined) {
                                    deviceTypesInZone.VIDEOpb.push(pbConfig);
                                }
                                addZone = true;
                                break;
                            case dSS.config.groups.JOKER:
                                deviceTypesInZone.JOKER = true;
                                joker = dSS.util.getJokerGroup(
                                                        device.get('groups'));
                                if (joker === 1) {
                                    deviceTypesInZone.LIGHT = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.LIGHTpb.push(
                                                pbConfig);
                                    }
                                } else if (joker === 2) {
                                    deviceTypesInZone.SHADOW = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.SHADOWpb.push(
                                                pbConfig);
                                    }
                                } else if (joker === 3) {
                                    deviceTypesInZone.CLIMATE = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.CLIMATEpb.push(
                                                pbConfig);
                                    }
                                } else if (joker === 4) {
                                    deviceTypesInZone.AUDIO = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.AUDIOpb.push(
                                                pbConfig);
                                    }
                                } else if (joker === 5) {
                                    deviceTypesInZone.VIDEO = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.VIDEOpb.push(
                                                pbConfig);
                                    }
                                } else if (joker === 9) {
                                    deviceTypesInZone.WHITE = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.WHITEpb.push(
                                                pbConfig);
                                    }
                                }
                                addZone = true;
                                break;
                            case dSS.config.groups.WHITE:
                                deviceTypesInZone.WHITE = true;
                                if (pbConfig !== undefined) {
                                    deviceTypesInZone.WHITEpb.push(pbConfig);
                                }

                                addZone = true;
                                break;
                            default:
                                break;
                        }
                    }
                });
                if (addZone === true) {
                    nodes.push({
                        text: record.get('name'),
                        id: record.get('id'),
                        leaf: false,
                        singleClickExpand: true,
                        children: this.generateSubNodes(record.get('id'),
                                                        deviceTypesInZone)
                    });
                }
            }
            return true;
        },
        this);

        var root = new Ext.tree.AsyncTreeNode({
            children: nodes,
            id: 'root',
            singleClickExpand: true,
            leaf: false
        });
        this.treePanel.setRootNode(root);

        if (selectedPath.length > 0) {
            panel.selectPath(selectedPath, null, function(result, node) {
                if (result === false) {
                    if (root.firstChild !== null) {
                        root.firstChild.select();
                    }
                }
            });
        } else {
            if (root.firstChild !== null) {
                root.firstChild.select();
            }
        }

        if (selModel.getSelectedNode() !== null) {
            this.handleNodeClick(selModel.getSelectedNode());
        }

        if (this.activeNode !== undefined) {
            this.activeNode = panel.getNodeById(this.activeNode.id);
            if (this.activeNode !== undefined) {
                this.activeNode.setCls('active-scene-node');
            }
        }

        this.dirtyTree = false;
    },

    handleNodeClick: function(node) {
        if (!this.isVisible()) {
            return;
        }

        if (node === null) {
            return;
        }

        if (node.isLeaf() === true) {
            this.menuPanel.getTopToolbar().get('callSceneButton').enable();
            this.menuPanel.getTopToolbar().get('saveSceneButton').enable();
            if (node.attributes.undo === true) {
                this.menuPanel.getTopToolbar().get('undoSceneButton').enable();
            } else {
                this.menuPanel.getTopToolbar().get('undoSceneButton').disable();
            }
        } else {
            this.menuPanel.getTopToolbar().get('callSceneButton').disable();
            this.menuPanel.getTopToolbar().get('saveSceneButton').disable();
            this.menuPanel.getTopToolbar().get('undoSceneButton').disable();
        }

        // we are on the zone node
        var path = node.getPath().split('/').slice(1);
        if (path.length === 4) {
            var ids = this.getZoneSceneGroup(node.id);
            if (ids.groupId !== null) {
                this.menuPanel.getTopToolbar().get('renameSceneButton').enable();
            } else {
                this.menuPanel.getTopToolbar().get('renameSceneButton').disable();
            }
            dSS.MessageBus.fireEvent('sceneselected', parseInt(path[3].split('-')[2]));
        } else {
            this.menuPanel.getTopToolbar().get('renameSceneButton').disable();
            dSS.MessageBus.fireEvent('sceneselected', 0);
        }

        if (path.length < 2) {
            return;
        }

        var store = dSS.data.Loader.getDeviceStore();
        store.clearFilter();
        store.filterBy(function(record) {
            var zoneId = parseInt(path[1], 10);

            if (record.get('isPresent') === false) {
                return false;
            }

            if (zoneId === 0) {
                return true;
            }

            if (record.get('zoneId') === zoneId) {
                if (path.length === 2) {
                    return true;
                }

                var fid = dSS.util.parseFunctionID(record.get('functionID'));

                var nodeType = path[2].split('-');
                // all leaf node ids will have 3 components: zone-group-scene
                if (nodeType.length < 3) {
                    return false;
                }

                // if the group is set, we sort only by group
                if (nodeType[1] !== 'all') {
                    var group = parseInt(nodeType[1], 10);
                    if (!isNaN(group)) {
                        if (fid[0] === group) {
                            return true;
                        }

                        if ((fid[0] === dSS.config.groups.JOKER) &&
                            (dSS.util.getJokerGroup(record.get('groups')) ===
                             group)) {
                            return true;
                        }

                        return false;
                    }
                } else {
                    return true;
                }

                // at this stage groups do not matter, scene filtering applies
                var scene = parseInt(nodeType[2], 10);

                if (isNaN(scene)) {
                    return false;
                }
            }
            return false;
        });
    },
    buildTopToolbar: function() {
        return [
            {
                iconCls: 'editIcon',
                handler: this.renameSelectedScene.createDelegate(this),
                tooltip: dSS._("Rename selected scene"),
                id: 'renameSceneButton',
                disabled: true,
                scope: this
            },
            {
                iconCls: 'saveScene',
                handler: this.saveSelectedScene.createDelegate(this),
                id: 'saveSceneButton',
                tooltip: dSS._("Save selected scene"),
                disabled: true,
                scope: this
            },
            '-',
            {
                iconCls: 'callSceneIcon',
                handler: this.processSelectedScene.createDelegate(this,[false]),
                id: 'callSceneButton',
                tooltip: dSS._("Call selected scene"),
                disabled: true,
                scope: this
            },
            {
                iconCls: 'undoSceneIcon',
                handler: this.processSelectedScene.createDelegate(this,[true]),
                id: 'undoSceneButton',
                tooltip: dSS._("Undo selected scene"),
                disabled: true,
                scope: this
            },
            '->',
            {
                iconCls: 'showAllScenesIcon',
                handler: this.showAllScenes.createDelegate(this),
                tooltip: dSS._("Display all available scenes"),
                enableToggle: true,
                scope: this
            }
        ];
    },
    showAllScenes: function(button) {
        if (button.pressed === true) {
            this.disableSceneFilter = true;
        } else {
            this.disableSceneFilter = false;
        }
        this.markDirty();
    },
    getZoneSceneGroup: function(id) {
        var ret = {
            "zoneId": null,
            "sceneNumber": null,
            "groupId": null
        };

        var ids = id.split('-');
        if (ids.length < 3) {
            return ret;
        }

        var zoneId = parseInt(ids[0], 10);
        if (isNaN(zoneId)) {
            return ret;
        }

        ret.zoneId = zoneId;

        var groupId = parseInt(ids[1], 10);
        if (isNaN(groupId)) {
            groupId = null;
        }

        ret.groupId = groupId;

        var sceneNumber = parseInt(ids[2], 10);
        if (isNaN(sceneNumber)) {
            sceneNumber = null;
        }

        ret.sceneNumber = sceneNumber;

        return ret;
    },

    getSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();
        if ((node === null) || (!node.isLeaf())) {
            throw dSS._("Can not identify scene: no scene selected");
        }

        var ids = this.getZoneSceneGroup(node.id);
        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            throw dSS._("Can not identify scene: invalid data");
        }

        return ids.sceneNumber;
    },

    processSelectedScene: function(undo) {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();

        if (node === null) {
            sel = this.apartmentTreePanel.getSelectionModel();
            node = sel.getSelectedNode();
        }

        if ((node === null) || (!node.isLeaf())) {
            if (undo === true) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No scene selected, please select the scene that you would like to undo"));
            } else {
                Ext.Msg.alert(dSS._("Error"), dSS._("No scene selected, please select the scene that you would like to call"));
            }
            return;
        }

        this.processScene(node, undo);
    },
    processScene: function(node, undo) {
        if ((node === null) || (!node.isLeaf())) {
            return;
        }

        if ((undo === true) && (node.attributes.undo !== true)) {
            Ext.Msg.alert(dSS._("Error"),
                          dSS._("Undo is not available for this scene"));
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            return;
        }

        if (this.activeNode !== undefined) {
            this.activeNode.setCls();
        }

        node.setCls('active-scene-node');
        this.activeNode = node;
        if (undo === true) {
            dSS.util.undoSceneOnZone(ids.zoneId, ids.sceneNumber);
        } else {
            dSS.util.callSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId,
                                     true);
        }
    },
    saveSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();

        if (node === null) {
            sel = this.apartmentTreePanel.getSelectionModel();
            node = sel.getSelectedNode();
        }

        if ((node === null) || (!node.isLeaf())) {
            Ext.Msg.alert(dSS._("No scene selected"), dSS._("No scene selected, please select the scene that you would like to save"));
            return;
        }

        this.saveScene(node);
    },
    saveScene: function(node) {
        if ((node === null) || (!node.isLeaf())) {
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            return;
        }

        if (ids.sceneNumber !== 0) {
            if ((this.activeNode !== undefined) && (node !== this.activeNode)) {
                this.warnSaveDifferentScene(node, this.activeNode);
                return;
            }
            if (this.activeNode !== undefined) {
                this.activeNode.setCls();
            }
            node.setCls('active-scene-node');
            this.activeNode = node;

            dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId);
        } else {
            this.warnSaveScene0(node);
        }
    },
    warnSaveDifferentScene: function(node, activeNode) {
        var ids = this.getZoneSceneGroup(node.id);
        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            return;
        }

        var aIds = this.getZoneSceneGroup(activeNode.id);
        if ((aIds.zoneId === null) || (aIds.sceneNumber === null)) {
            return;
        }

        var calledName = dSS._(dSS.config.sceneNames[aIds.sceneNumber]) +
            this.getSceneName(aIds.zoneId, aIds.groupId, aIds.sceneNumber);

        var savingName = dSS._(dSS.config.sceneNames[ids.sceneNumber]) +
            this.getSceneName(ids.zoneId, ids.groupId, ids.sceneNumber);

        var title = dSS.gt.strargs(dSS._("Save scene %1%2%3"),
            [
                '&quot;', savingName, '&quot;'
            ]);

        var msg = dSS._("Called scene") + ': &quot;' + calledName +
                        '&quot;<br/>' +
                  dSS._("Saving scene") + ': &quot;' + savingName +
                        '&quot;<br/><br/>' +
                  dSS._("You are about to save a scene that is different " +
                        "from the called scene.") + '<br/>' +
                  dSS._("Are you sure that you want to overwrite the scene?");

        Ext.Msg.confirm(
            title,
            msg,
            function(btn) {
                if (btn === 'yes') {
                    if (this.activeNode !== undefined) {
                        this.activeNode.setCls();
                    }
                    node.setCls('active-scene-node');
                    this.activeNode = node;

                    dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber,
                                             ids.groupId);
                }
            },
            this
        );
    },
    warnSaveScene0: function(node) {
        var ids = this.getZoneSceneGroup(node.id);
        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            return;
        }

        var savingName = dSS._(dSS.config.sceneNames[ids.sceneNumber]) +
            this.getSceneName(ids.zoneId, ids.groupId, ids.sceneNumber);

        var title = dSS.gt.strargs(dSS._("Save scene %1%2%3"),
            [
                '&quot;', savingName, '&quot;'
            ]);

        var msg = dSS._("You are about to save scene 0 which represents " +
                        "the off scene. Note that you might not be able to " +
                        "switch off all devices in case some devices in the " +
                        "selected room are turned on right now.") + '<br/>' +
                  dSS._("Are you sure you want to save the scene?");

        Ext.Msg.confirm(
            title,
            msg,
            function(btn) {
                if (btn === 'yes') {
                    if (this.activeNode !== undefined) {
                        this.activeNode.setCls();
                    }
                    node.setCls('active-scene-node');
                    this.activeNode = node;

                    dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber,
                                             ids.groupId);
                }
            },
            this
        );
    },
    renameSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();

        if (node === null) {
            sel = this.apartmentTreePanel.getSelectionModel();
            node = sel.getSelectedNode();
        }

        if ((node === null) || (!node.isLeaf())) {
            Ext.Msg.alert(dSS._("Error"), dSS._("No scene selected, please select the scene that you want to rename"));
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId === null) ||
            (ids.groupId === null) || (ids.sceneNumber === null)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Could not determine scene from node"));
            return;
        }

        var sceneStore = dSS.data.Loader.getSceneStore();
        var recId = ids.zoneId + '-' + ids.groupId + '-' + ids.sceneNumber;

        var record = sceneStore.getById(recId);

        // this should never be the case, however if by any chance we missed
        // something I prefer having a messagebox here instead of some
        // random error
        if (record === undefined) {
            record = new sceneStore.recordType({
                                                    name : "",
                                                    scene: ids.sceneNumber,
                                                    group: ids.groupId,
                                                    zone: ids.zoneId
                                                }, recId);
            sceneStore.add([record]);
        }

        Ext.Msg.prompt(dSS._("Rename scene"), dSS._("New name for scene"),
                function(btn, text) {
                if ((btn === 'ok') && (text !== record.get('name'))) {
                    Ext.Ajax.request({
                        url: '/json/zone/sceneSetName',
                        disableCaching: true,
                        method: "GET",
                        params: {
                            id: ids.zoneId,
                            groupID: ids.groupId,
                            sceneNumber: ids.sceneNumber,
                            newName: text
                        },
                        success: function(result, request) {
                            try {
                                var jsonData = Ext.util.JSON.decode(result.responseText);
                                if (jsonData.ok) {
                                    var old = record.get('name');
                                    record.set('name', text);
                                    sceneStore.commitChanges();
                                    dSS.MessageBus.fireEvent('statusmessage',
                                        dSS._("Scene") + ' ' +
                                        dSS.util.getName(old) + ' ' +
                                        ' ' + dSS._("was renamed") + '.' +
                                        dSS._("New name") + ': ' + text);
                                } else {
                                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not rename scene"));
                                }
                            }
                            catch(err) {
                                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not rename scene") + ': ' + err);
                            }
                        },
                        failure: function(result, request) {
                            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not rename scene"));
                        }
                    });
                }
            },
            this, false, record.get('name'));
    }
});

Ext.reg('dssscenespanel', dSS.ScenesPanel);

